<?php
/**
 * Plugin Name: Club WPress Updater
 * Plugin URI: https://clubwpress.net
 * Description: Automatic updates for plugins and themes from ClubWPress.net
 * Version: 1.2.8
 * Tested up to: 6.2
 * Author: ClubWPress.net
 * Author URI: https://clubwpress.net
 * Text Domain: clubwpress-updater
 */
 
 if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

if ( ! class_exists( 'CLUBWORDPRESS' ) ) {

    global $wpdb;

	define('CLUBWORDPRESS_TBL_KEY', $wpdb->prefix . "cw_articles_tbl");
	define('CLUBWORDPRESS_DB_VERSION', '1.39');
	define('CLUBWORDPRESS_VERSION', '1.2.8');

	/**
	 * Main CLUBWORDPRESS Class
	 * @class CLUBWORDPRESS
	 * @version	1.0
	 */
	final class CLUBWORDPRESS {
		
		protected static $_instance = null;

		public $program = null;
		public $plugin_slug = 'clubwpress-updater';
		public static function instance() {
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self();
			}
			return self::$_instance;
		}
		
		public function __construct() {
			
			$this->includes();
			$this->init_hooks();
			
		  
            $upload = wp_upload_dir();
            $upload_dir = $upload['basedir'];
            $upload_dir = $upload_dir . '/cw_backups';
            if (! is_dir($upload_dir)) {
                mkdir( $upload_dir, 0755);
            } else {
                if( @chmod($upload_dir, 0700) ) {
                    @chmod($upload_dir, 0755);
                }
            }
	       do_action( 'cw_loaded' );
		}

		public function init_hooks() {
			add_action( 'init', array( $this, 'load_plugin_textdomain' ), 0 );
			add_action( 'init', array( $this, 'auto_update' ), 10 );
			add_action('plugins_loaded', array( $this, 'cw_plugins_loaded_handler' ), 0 );
		}

		function cw_plugins_loaded_handler() {
		    if (is_admin()) {
		        if (!$this->cw_table_exists() || get_option('cw_db_version') != CLUBWORDPRESS_DB_VERSION) {
		            require_once(dirname(__FILE__) . '/includes/clubwp-install.php');
		        }
		    }
		}

		public function includes() {
            include_once( 'includes/clubwp-plugin-activation.php' );
            include_once( 'includes/clubwp-notices.php' );
            include_once( 'includes/clubwp-menu.php' );
            include_once( 'includes/clubwp-cleaner-hooks.php' );
			if ( ! class_exists( 'CwSlmClient' ) ) {
				include_once( 'includes/clubwp-client.php' );
			}
		}
        
        public function auto_update() {
            include_once ( 'includes/clubwp-auto-update.php' );
        }

		public function load_plugin_textdomain() {
			$domain = $this->plugin_slug;
			load_plugin_textdomain( $domain, false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' ); 
		}

		public static function install() {
			if (! wp_next_scheduled ( 'clubwordpress_daily_update' )) {
				wp_schedule_event(time(), 'daily', 'clubwordpress_daily_update');
		    }
		}

		public static function uninstall() {
			wp_clear_scheduled_hook('clubwordpress_daily_update');
            delete_option('cw_db_version' );
		}

        public function cw_table_exists() {
            global $wpdb;
            $table_name = CLUBWORDPRESS_TBL_KEY;
            $query = $wpdb->prepare( 'SHOW TABLES LIKE %s', $wpdb->esc_like( $table_name ) );
            if ( $wpdb->get_var( $query ) != $table_name ) {
                return false;
            }else{
                return true;
            }
        }
	}
}

register_activation_hook( __FILE__, array( 'CLUBWORDPRESS', 'install' ) );
register_deactivation_hook(__FILE__, array( 'CLUBWORDPRESS', 'uninstall' ) );

function CWSCN() {
	return CLUBWORDPRESS::instance();
}

// Global for backwards compatibility.
$GLOBALS['clubwordpress'] = CWSCN();