<?php
/**
 * Plugin installation and activation for Club WordPress .
 */


if ( ! class_exists( 'CW_Plugin_Activation' ) ) {
	class CW_Plugin_Activation {
		const CWPM_VERSION = '2.6.1';
		const WP_REPO_REGEX = '|^http[s]?://wordpress\.org/(?:extend/)?plugins/|';
		const IS_URL_REGEX = '|^http[s]?://|';
		public static $instance;
		public $plugins = array();
		public $wp_version;
		public function __construct() {
			$this->wp_version = $GLOBALS['wp_version'];
			do_action_ref_array( 'cwpm_init', array( $this ) );
            add_action( 'wp_loaded', array( $this, 'init' ) );		
            add_action( 'wp_loaded', array( $this, 'plugins_update_hooks' ) );
		}

        public function init() {
                if (is_admin()) {
                      add_filter('site_transient_update_plugins',   array( $this,'check_plugins_update'), 9999999);
                      add_filter('site_transient_update_themes',   array( $this,'check_themes_update'), 9999999);
                      add_filter('site_transient_update_plugins',  array( $this,'cw_disable_check_updater' ), 9999998);
                      add_filter('site_transient_update_themes',   array( $this,'cw_disable_check_updater_themes') , 9999998);
                }
        }
        
		public function plugins_update_hooks() {
            global $cw_tmp_link;
            $installed_plugins = $this->get_plugins();
            $installed_plugins_fp = $this->get_installed_cw_plugins(array_keys($installed_plugins));
            if($installed_plugins_fp){
                foreach($installed_plugins_fp as $plugin){
                    $slug = $plugin->filepath;
                    $cw_tmp_link = '';
                    if(isset($plugin->permalink)){
                        $cw_tmp_link = $plugin->permalink;
                    }
                    if(stristr($slug, '/')){
                         add_action( 'in_plugin_update_message-'.$slug, function( $data, $response ) {
                            global $cw_tmp_link;
                            echo '<br> ';
                            printf(
                                __('<a href="%s"><strong>Update with Club WordPress Updater</strong></a>', 'clubwpress-updater'),
                                admin_url('/admin.php?page=cw_updater_settings#cw_filter-installed|1'),
                                $cw_tmp_link
                            );
                        }, 10, 2 );
                    }
                }
            }
        }
            //
		public function check_plugins_update($transient) {
            global $cw_manual_update, $package;
            $installed_plugins = $this->get_plugins();
            $installed_plugins_fp = $this->get_installed_cw_plugins(array_keys($installed_plugins));
            if($installed_plugins_fp){
                foreach($installed_plugins_fp as $plugin){
                    $slug = $plugin->filepath;
                    $permalink = '';
                    if(isset($plugin->permalink)){
                        $permalink = $plugin->permalink;
                    }
                    if(isset($installed_plugins[$slug])){
                        $current_version = $installed_plugins[$slug]['Version'];
                        $remote_version = $plugin->version;
                        if(isset($transient->response[$slug]->package)){
                            $package = $transient->response[$slug]->package;
                        }

						//Version syntax !
                        if ( !stristr( $remote_version , '+' ) && (version_compare( $current_version , $remote_version, '<' ) || $package != '')) {
                            $obj = new stdClass();
                            $obj->slug = $slug;
                            $obj->new_version = $remote_version;
                            $obj->url = $permalink;
                            $obj->plugin = $slug;
                            $obj->package = $package;
							if(isset($transient->response)){
								$transient->response[$slug] = $obj;
							}
                            
                        }
                    }
                }
            }
            return $transient; 
        }
        
        public function check_themes_update($transient) {
            global $cw_manual_update, $package;
            $installed_themes = wp_get_themes();
            $installed_themes_fp = $this->get_installed_cw_themes(array_keys($installed_themes));
            if($installed_themes_fp){
                foreach($installed_themes_fp as $theme){
                    $slug = $theme->filepath;
                    $short_slug = explode('/', $slug);
                    $short_slug = $short_slug[0];
                    if(isset($installed_themes[$short_slug])){
                        $current_version = $installed_themes[$short_slug]['Version'];
                        $remote_version = $theme->version;
                            if(isset($transient->response[$slug]->package)){
                                $package = $transient->response[$slug]->package;
                            }
                            if ( version_compare( $current_version , $remote_version, '<' ) || $package != '') {
                            $obj = array();
                            $obj['new_version'] = $remote_version;
                            $obj['url'] = '';
                            $obj['theme'] = $short_slug;
                            $obj['package'] = $package;
							if(isset($transient->response[$short_slug])){
								$transient->response[$short_slug] = $obj;
							}
                            
                        }
                    }
                }
            }
            return $transient; 
        }
        
        public  function cw_disable_check_updater( $transient ) {
            $installed_plugins = $this->get_plugins();
            $installed_plugins_fp = $this->get_installed_cw_plugins(array_keys($installed_plugins));
            if($installed_plugins_fp){
                foreach($installed_plugins_fp as $plugin){
                    $slug = $plugin->filepath;
                    if(isset($installed_plugins[$slug])){
                        if(isset($transient->response[$slug])){
                           unset( $transient->response[$slug] );  
                        }
                    }
                }
            }
             return $transient;
        }

       public function cw_disable_check_updater_themes( $transient ) {
            
            $installed_themes = wp_get_themes(); 
            $installed_themes_fp = $this->get_installed_cw_themes(array_keys($installed_themes));
            if($installed_themes_fp){
                foreach($installed_themes_fp as $theme){
                    $slug = $theme->filepath;
                    $short_slug = explode('/', $slug);
                    $short_slug = $short_slug[0];
                    if(isset($installed_themes[$short_slug])){
                         if(isset($transient->response[$slug])){
                           unset( $transient->response[$slug] );  
                        }
                    }
                }
            }
            return $transient;
        }

		public function do_plugin_install() {
           global $cw_install_output;
			if ( empty( $_GET['plugin'] ) || empty( $_GET['type'] )) {
				return false;
			}
			$slug = $this->sanitize_key( urldecode( $_GET['plugin'] ) );
			$item_type = $_GET['type'];
            $source = (isset($_GET['source']) ? $_GET['source'] : false);
            $version = (isset($_GET['version']) ? $_GET['version'] : false);
            $item_name = (isset($_GET['item_name']) ? $_GET['item_name'] : false);
			if ( ( isset( $_GET['cwpm-install'] ) && 'install-plugin' === $_GET['cwpm-install'] ) || ( isset( $_GET['cwpm-update'] ) && 'update-plugin' === $_GET['cwpm-update'] ) || ( isset( $_GET['cwpm-restore'] ) && 'restore-plugin' === $_GET['cwpm-restore'] )) {
				$install_type = 'install';
				if ( isset( $_GET['cwpm-update'] ) && 'update-plugin' === $_GET['cwpm-update'] ) {
					$install_type = 'update';
				}elseif ( isset( $_GET['cwpm-restore'] ) && 'restore-plugin' === $_GET['cwpm-restore'] ) {
					$install_type = 'restore';
				}
               if(!wp_verify_nonce($_GET['cwpm-nonce'], 'cwpm-' . $install_type)){
                return false;
               }
				if ( ! class_exists( 'Plugin_Upgrader', false ) ) {
					require_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';
				}
			   if ( ! class_exists( 'Plugin_Upgrader_Skin', false ) ) {
					require_once ABSPATH . 'wp-admin/includes/class-wp-upgrader-skin.php';
				}
				$url = wp_nonce_url(
					add_query_arg(
						array(
							'plugin'                 => urlencode( $slug ),
							'cwpm-' . $install_type => $install_type . '-plugin',
						),
						''
					),
					'cwpm-' . $install_type,
					'cwpm-nonce'
				);
                if (empty($wp_filesystem)) {
                    require_once (ABSPATH . '/wp-admin/includes/file.php');
                    WP_Filesystem();
                }
                if (!function_exists('show_message')) {
                    require_once (ABSPATH . '/wp-admin/includes/misc.php');
                }
				$method = ''; 
				if ( false === ( $creds = request_filesystem_credentials( esc_url_raw( $url ), $method, false, false, array() ) ) ) {
					return true;
				}
				if ( ! WP_Filesystem( $creds ) ) {
					request_filesystem_credentials( esc_url_raw( $url ), $method, true, false, array() ); // Setup WP_Filesystem.
					return true;
				}
				$extra         = array();
				$extra['slug'] = $slug;
				$api =  null;
                $skin_arg_type = ($install_type == 'restore' ? 'bundled' : 'web');
                if($item_type == 'plugin'){
                        $url = add_query_arg(
                            array(
                                'action' => $install_type . '-plugin',
                                'plugin' => urlencode( $slug ),
                            ),
                            'update.php'
                        );
                        $skin_args = array(
                            'type'   => $skin_arg_type,
                            'title'  => $item_name,
                            'url'    => esc_url_raw( $url ),
                            'nonce'  => $install_type . '-plugin_' . $slug,
                            'plugin' => '',
                            'api'    => $api,
                            'extra'  => $extra,
                        );
                        if ( 'update' === $install_type || 'restore' === $install_type ) {
                            $skin_args['plugin'] = $_GET['filepath'];
                            $skin                = new Plugin_Upgrader_Skin( $skin_args );
                        } else {
                            $skin = new Plugin_Installer_Skin( $skin_args );
                        }
                         $upgrader = new Plugin_Upgrader();
                        //add_filter( 'upgrader_source_selection', array( $this, 'maybe_adjust_source_dir' ), 1, 3 );
                   }else{
                     $url = add_query_arg(
                        array(
                            'action' => $install_type . '-theme',
                            'theme' => urlencode( $slug ),
                        ),
                        'update.php'
                    );
                    if ( !class_exists( 'Theme_Upgrader', false ) ) {
                        require_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';
                    }
                    if ( !class_exists( 'Theme_Upgrader_Skin', false ) ) {
                        require_once ABSPATH . 'wp-admin/includes/class-wp-upgrader-skin.php';
                    }
                    $skin_args = array(
                        'type'   => $skin_arg_type,
                        'title'  => $item_name,
                        'url'    => esc_url_raw( $url ),
                        'nonce'  => $install_type . '-theme_' . $slug,
                        'theme' => '',
                        'api'    => $api,
                        'extra'  => $extra,
                    );
                    if ( 'update' === $install_type || 'restore' === $install_type) {
                        $skin_args['theme'] = $_GET['filepath'];
                        $skin                = new Theme_Upgrader_Skin( $skin_args );
                    } else {
                        $skin = new Theme_Installer_Skin( $skin_args );
                    }
                     $upgrader = new Theme_Upgrader();
                }
                
				if ( 'update' === $install_type || 'restore' === $install_type) {
                    global $cw_manual_update, $package;
                    $cw_manual_update = true;
                     if($item_type == 'plugin'){
                        $reactivate = $this->is_plugin_active($_GET['filepath']);
                        $to_inject  = array();
                        $to_inject[ $slug ]['file_path'] = $_GET['filepath'];
                        $to_inject[ $slug ]['source'] = $source;
                        $to_inject[ $slug ]['version'] = $version;
                        $this->inject_plugin_update_info( $to_inject );
                        $package = $source;
                        $upgraded = $upgrader->upgrade( $_GET['filepath'] );
                     }else{
                        $reactivate = $this->is_theme_active($slug);
                        $to_inject  = array();
                        $to_inject[ $slug ]['file_path'] = $_GET['filepath'];
                        $to_inject[ $slug ]['source'] = $source;
                        $to_inject[ $slug ]['version'] = $version;
                        $this->inject_theme_update_info( $to_inject );
                        $package = $source;
                        $upgraded = $upgrader->upgrade($slug);
                     }
                    
                    if($reactivate){
                        
                        	if ( $item_type == 'plugin' && false === $this->activate_single_plugin( $_GET['filepath'], $slug ) ) {
                                
                            }elseif( $item_type == 'theme' && false === $this->activate_single_theme( $slug ) ) {
                               
                            }
                    }
                    $cw_manual_update = false;
				} else {     
				  $installed = $upgrader->install( $source );
				}
                wp_clean_plugins_cache( true );
                echo '<---->';
				//remove_filter( 'upgrader_source_selection', array( $this, 'maybe_adjust_source_dir' ), 1 );
                if($item_type == 'plugin'){
                   // $this->populate_file_path( $slug );
				}
				return true;
			} elseif (isset( $_GET['filepath'], $_GET['cwpm-activate'] ) && 'activate-plugin' === $_GET['cwpm-activate'] ) {
                if(!wp_verify_nonce($_GET['cwpm-nonce'], 'cwpm-activate')){
                    return false;
               }
				if ( $item_type == 'plugin' && false === $this->activate_single_plugin( $_GET['filepath'], $slug ) ) {
					return true; 
				}elseif( $item_type == 'theme' && false === $this->activate_single_theme( $slug ) ) {
					return true; 
				}
			}
			return false;
		}
        
        function install_plugin( $plugin_zip ) {
            if ( ! class_exists( 'Plugin_Upgrader', false ) ) {
                include_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';
            }
            $upgrader = new Plugin_Upgrader();
            $installed = $upgrader->install( $plugin_zip );
            wp_clean_plugins_cache( false );
          return $installed;
        }

        function upgrade_plugin( $plugin_slug ) {
           if ( ! class_exists( 'Plugin_Upgrader', false ) ) {
                include_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';
            }
          $upgrader = new Plugin_Upgrader();
          $upgraded = $upgrader->upgrade( $plugin_slug );
          wp_clean_plugins_cache( false );
          return $upgraded;
        }

		public function inject_plugin_update_info( $plugins ) {
			$repo_updates = get_site_transient( 'update_plugins' );
			if ( ! is_object( $repo_updates ) ) {
				$repo_updates = new stdClass;
			} 
			foreach ( $plugins as $slug => $plugin ) {
				$file_path = $plugin['file_path'];
				if ( empty( $repo_updates->response[ $file_path ] ) ) {
					$repo_updates->response[ $file_path ] = new stdClass;
				}
				$repo_updates->response[ $file_path ]->slug        = $slug;
				$repo_updates->response[ $file_path ]->plugin      = $file_path;
				$repo_updates->response[ $file_path ]->new_version = $plugin['version'];
				$repo_updates->response[ $file_path ]->package     = $plugin['source'];
				if ( empty( $repo_updates->response[ $file_path ]->url ) && ! empty( $plugin['external_url'] ) ) {
					$repo_updates->response[ $file_path ]->url = $plugin['external_url'];
				}
			}
			set_site_transient( 'update_plugins', $repo_updates);
		}
        
        public function inject_theme_update_info( $themes ) {
			$repo_updates = get_site_transient( 'update_themes' );
			if ( ! is_object( $repo_updates ) ) {
				$repo_updates = new stdClass;
			}
			foreach ( $themes as $slug => $theme ) {
				if ( empty( $repo_updates->response[ $slug ] ) ) {
					$repo_updates->response[ $slug ] = array();
				}
				$repo_updates->response[ $slug ]['theme']     = $slug;
				$repo_updates->response[ $slug ]['new_version'] = $theme['version'];
				$repo_updates->response[ $slug ]['package']     = $theme['source'];
			}
			set_site_transient( 'update_themes', $repo_updates );
		}
        
		public function maybe_adjust_source_dir( $source, $remote_source, $upgrader ) {
            global $wp_filesystem;
			if ( ! is_object( $wp_filesystem ) ) {
				return $source;
			}
			$source_files = array_keys( $wp_filesystem->dirlist( $remote_source ) );
			if ( 1 === count( $source_files ) && false === $wp_filesystem->is_dir( $source ) ) {
				return $source;
			}
			$desired_slug = '';
			if ( false === $upgrader->bulk && ! empty( $upgrader->skin->options['extra']['slug'] ) ) {
				$desired_slug = $upgrader->skin->options['extra']['slug'];
			} else {
				foreach ( $this->plugins as $slug => $plugin ) {
					if ( ! empty( $upgrader->skin->plugin_names[ $upgrader->skin->i ] ) && $plugin['name'] === $upgrader->skin->plugin_names[ $upgrader->skin->i ] ) {
						$desired_slug = $slug;
						break;
					}
				}
				unset( $slug, $plugin );
			}

			if ( ! empty( $desired_slug ) ) {
				$subdir_name = untrailingslashit( str_replace( trailingslashit( $remote_source ), '', $source ) );
				if ( ! empty( $subdir_name ) && $subdir_name !== $desired_slug ) {
					$from_path = untrailingslashit( $source );
					$to_path   = trailingslashit( $remote_source ) . $desired_slug;
					if ( true === $wp_filesystem->move( $from_path, $to_path ) ) {
						return trailingslashit( $to_path );
					} else {
						return new WP_Error( 'rename_failed', esc_html__( 'The remote plugin package does not contain a folder with the desired slug and renaming did not work.', 'cluwordpress-updater' ) . ' ' . esc_html__( 'Please contact the plugin provider and ask them to package their plugin according to the WordPress guidelines.', 'cluwordpress-updater' ), array( 'found' => $subdir_name, 'expected' => $desired_slug ) );
					}
				} elseif ( empty( $subdir_name ) ) {
					return new WP_Error( 'packaged_wrong', esc_html__( 'The remote plugin package consists of more than one file, but the files are not packaged in a folder.', 'cluwordpress-updater' ) . ' ' . esc_html__( 'Please contact the plugin provider and ask them to package their plugin according to the WordPress guidelines.', 'cluwordpress-updater' ), array( 'found' => $subdir_name, 'expected' => $desired_slug ) );
				}
			}
			return $source;
		}

		protected function activate_single_plugin( $file_path, $slug, $automatic = false ) {
			if ( $this->can_plugin_activate( $slug ) ) {
                global $cw_install_output;
				$activate = activate_plugin( $file_path );
				if ( is_wp_error( $activate ) ) {
					return false; 
				} else {
					return true; 
				}
			} 
			return false;
		}

	
		protected function activate_single_theme( $slug ) {
			if ( $this->is_theme_installed( $slug ) && !$this->is_theme_active($slug)) {
				$activate = switch_theme( $slug );
				if ( is_wp_error( $activate ) ) {
					return false; 
				} else {
                    return true;	
				}
			} 
			return false;
		}
		
		public function sanitize_key( $key ) {
			$raw_key = $key;
			$key     = preg_replace( '`[^A-Za-z0-9_-]`', '', $key );
			return apply_filters( 'cwpm_sanitize_key', $key, $raw_key );
		}

	
		public function populate_file_path( $plugin_slug = '' ) {
			if ( ! empty( $plugin_slug ) && is_string( $plugin_slug ) && isset( $this->plugins[ $plugin_slug ] ) ) {
				$this->plugins[ $plugin_slug ]['file_path'] = $this->_get_plugin_basename_from_slug( $plugin_slug );
			} else {
				foreach ( $this->plugins as $slug => $values ) {
					$this->plugins[ $slug ]['file_path'] = $this->_get_plugin_basename_from_slug( $slug );
				}
			}
		}

		protected function _get_plugin_basename_from_slug( $slug ) {
			$keys = array_keys( $this->get_plugins() );
			foreach ( $keys as $key ) {
				if ( preg_match( '|^' . $slug . '/|', $key ) ) {
					return $key;
				}
			}
			return $slug;
		}

		protected function get_plugins_api( $slug ) {
			static $api = array(); 
			if ( ! isset( $api[ $slug ] ) ) {
				if ( ! function_exists( 'plugins_api' ) ) {
					require_once ABSPATH . 'wp-admin/includes/plugin-install.php';
				}
				$response = plugins_api( 'plugin_information', array( 'slug' => $slug, 'fields' => array( 'sections' => false ) ) );
				$api[ $slug ] = false;
				if ( is_wp_error( $response ) ) {
					wp_die( esc_html( $this->strings['oops'] ) );
				} else {
					$api[ $slug ] = $response;
				}
			}
			return $api[ $slug ];
		}

		public function is_plugin_installed( $slug ) {
			$installed_plugins = $this->get_plugins();
			return ( ! empty( $installed_plugins[ $slug ] ) );
		}
        
		public function is_theme_installed( $slug ) {
			$installed_themes = wp_get_themes(); 
			return ( ! empty( $installed_themes[ $slug ] ) );
		}
       
		public function is_plugin_active( $slug ) {
			return ( is_plugin_active($slug) );
		}
        
		public function is_theme_active( $slug ) {
            $current_theme = wp_get_theme();
            if($current_theme && $this->is_theme_installed($slug)){
                if($current_theme->stylesheet == $slug){ 
                    return true;
                }
            }
            return false;
		}

		public function can_plugin_update( $slug ) {
			if ( 'repo' !== $this->plugins[ $slug ]['source_type'] ) {
				return true;
			}
			$api = $this->get_plugins_api( $slug );
			if ( false !== $api && isset( $api->requires ) ) {
				return version_compare( $this->wp_version, $api->requires, '>=' );
			}
			return true;
		}

		public function is_plugin_updatetable( $slug ) {
			if ( ! $this->is_plugin_installed( $slug ) ) {
				return false;
			} else {
				return ( false !== $this->does_plugin_have_update( $slug ) && $this->can_plugin_update( $slug ) );
			}
		}

		public function can_plugin_activate( $slug ) {
			return ( ! $this->is_plugin_active( $slug ) && ! $this->does_plugin_require_update( $slug ) );
		}

		public function get_installed_version( $slug ) {
			$installed_plugins = $this->get_plugins();
			if ( ! empty( $installed_plugins[ $slug ] ) ) {
				return $installed_plugins[$slug ]['Version'];
			}
			return '';
		}
      
		public function get_installed_theme_version( $slug ) {
			$installed_themes = wp_get_themes();
			if ( ! empty( $installed_themes[ $slug ] ) ) {
				return $installed_themes[$slug ]['Version'];
			}
			return '';
		}
    

		public function does_plugin_require_update( $slug, $remote_file_version = false ) {
			$installed_version = $this->get_installed_version( $slug );
			$product   = $this->get_plugin_row( $slug );
            if($product){
                $minimum_version   =  ($remote_file_version ? $remote_file_version : $product->version);
			     return version_compare( $minimum_version, $installed_version, '>' );
            }
			return false;
		}
       
		public function does_theme_require_update( $slug, $remote_file_version = false ) {
            $short_slug = explode('/', $slug);
            $short_slug = $short_slug[0];
			$installed_version = $this->get_installed_theme_version( $short_slug );
			$product   = $this->get_plugin_row( $slug );
			$minimum_version   =  ($remote_file_version ? $remote_file_version : $product->version);
			return version_compare( $minimum_version, $installed_version, '>' );
		}
        
        public function get_plugin_row( $slug ) {
			 global $wpdb;
            $cw_table = CLUBWORDPRESS_TBL_KEY;
            $query = "SELECT * FROM " . $cw_table . " WHERE filepath = '%s'";
             $query = $wpdb->prepare( $query, $slug );
             $product = $wpdb->get_row( $query );
            if($product){
                return $product;
            }
		}
        
         public function get_installed_cw_plugins($installed_plugins ) {
			 global $wpdb;
            $cw_table = CLUBWORDPRESS_TBL_KEY;
            $query = "SELECT filepath, version  FROM " . $cw_table . " WHERE type = 'plugin' AND filepath IN ('".implode("','", $installed_plugins)."')";
            $installed_cw_plugins = $wpdb->get_results( $query );
            if($installed_cw_plugins){
                return $installed_cw_plugins;
            }
		}
        
        public function get_installed_cw_themes($installed_themes ) {
			global $wpdb;
            $cw_table = CLUBWORDPRESS_TBL_KEY;
            $new_installed_theme = array();
            foreach($installed_themes as $installed_theme){
               $new_installed_theme[] =  $installed_theme.'/style.css';
            }
            $query = "SELECT filepath, version  FROM " . $cw_table . " WHERE type = 'theme' AND filepath IN ('".implode("','", $new_installed_theme)."')";
            $installed_cw_themes = $wpdb->get_results( $query );
            if($installed_cw_themes){
                return $installed_cw_themes;
            }
		}
		
		public function does_plugin_have_update( $slug ) {
			if ( 'repo' !== $this->plugins[ $slug ]['source_type'] ) {
				if ( $this->does_plugin_require_update( $slug ) ) {
					return $this->plugins[ $slug ]['version'];
				}
				return false;
			}
			$repo_updates = get_site_transient( 'update_plugins' );
			if ( isset( $repo_updates->response[ $this->plugins[ $slug ]['file_path'] ]->new_version ) ) {
				return $repo_updates->response[ $this->plugins[ $slug ]['file_path'] ]->new_version;
			}
			return false;
		}

		public function get_plugins( $plugin_folder = '' ) {
			if ( ! function_exists( 'get_plugins' ) ) {
				require_once ABSPATH . 'wp-admin/includes/plugin.php';
			}
			return get_plugins( $plugin_folder );
		}
      
		public static function get_instance() {
			if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {
				self::$instance = new self();
			}
			return self::$instance;
		}
	}

	if ( !function_exists( 'load_cwpm_plugin_activation' ) ) {
	
		function load_cwpm_plugin_activation() {
			$GLOBALS['cwpm'] = CW_Plugin_Activation::get_instance();
		}
	}

	if ( did_action( 'plugins_loaded' ) ) {
		load_cwpm_plugin_activation();
	} else {
		add_action( 'plugins_loaded', 'load_cwpm_plugin_activation' );
	}
}